CONVERT_MS_TO_S = 0.001
KHZ_ISI_THRESHOLD = 0.0011  # 1Kz and above will be removed

SINGLE = "single"
DOUBLE = "double"
TRAIN = "train"

SD_MULTIPLIER = 3

TRAIN_SINGLE_PULSE = "train_single_pulse"
STUDY_INFO_FILE = "study_info.json"
REFLEX_FILE = "study_reflex.json"
SUBJECT_REFLEX_FILE = "subject_reflex.json"
